@@ -1,11 +1,25 @@
-#!/usr/bin/perl
-
+#!/usr/bin/perl -w
 use strict;
+
+use blib;
 use Fuse;
 use IO::File;
 use POSIX qw(ENOENT ENOSYS EEXIST EPERM O_RDONLY O_RDWR O_APPEND O_CREAT);
-use Fcntl qw(S_ISBLK S_ISCHR S_ISFIFO SEEK_SET);
-require 'syscall.ph'; # for SYS_mknod and SYS_lchown
+use Fcntl qw(S_ISBLK S_ISCHR S_ISFIFO SEEK_SET S_ISREG S_ISFIFO S_IMODE);
+my $can_syscall = eval {
+	require 'syscall.ph'; # for SYS_mknod and SYS_lchown
+};
+if (!$can_syscall && open my $fh, '<', '/usr/include/sys/syscall.h') {
+	my %sys = do { local $/ = undef;
+			<$fh> =~ m/\#define \s+ (\w+) \s+ (\d+)/gxms;
+        };
+	close $fh;
+	if ($sys{SYS_mknod} && $sys{SYS_lchown}) {
+		*SYS_mknod  = sub { $sys{SYS_mknod}  };
+		*SYS_lchown = sub { $sys{SYS_lchown} };
+		$can_syscall = 1;
+	}
+}
 
 sub fixup { return "/tmp/fusetest-" . $ENV{LOGNAME} . shift }
 
@@ -63,9 +77,8 @@ sub x_write {
 
 sub err { return (-shift || -$!) }
 
-sub x_readlink { return readlink(fixup(shift)                 ); }
-sub x_unlink { return unlink(fixup(shift)) ? 0 : -$!;          }
-sub x_rmdir { return err(rmdir(fixup(shift))               ); }
+sub x_readlink { return readlink(fixup(shift));         }
+sub x_unlink   { return unlink(fixup(shift)) ? 0 : -$!; }
 
 sub x_symlink { print "symlink\n"; return symlink(shift,fixup(shift)) ? 0 : -$!; }
 
@@ -77,6 +90,7 @@ sub x_rename {
 }
 sub x_link { return link(fixup(shift),fixup(shift)) ? 0 : -$! }
 sub x_chown {
+	return -ENOSYS() if ! $can_syscall;
 	my ($fn) = fixup(shift);
 	print "nonexistent $fn\n" unless -e $fn;
 	my ($uid,$gid) = @_;
@@ -100,11 +114,23 @@ sub x_mkdir { my ($name, $perm) = @_; return 0 if mkdir(fixup($name),$perm); ret
 sub x_rmdir { return 0 if rmdir fixup(shift); return -$!; }
 
 sub x_mknod {
+	return -ENOSYS() if ! $can_syscall;
 	# since this is called for ALL files, not just devices, I'll do some checks
 	# and possibly run the real mknod command.
 	my ($file, $modes, $dev) = @_;
 	$file = fixup($file);
-	$! = 0;
+	undef $!;
+	if ($^O eq 'freebsd' || $^O eq 'darwin' || $^O eq 'netbsd') {
+		if (S_ISREG($modes)) {
+			open(FILE, '>', $file) || return -$!;
+			print FILE "";
+			close(FILE);
+			return 0;
+		} elsif (S_ISFIFO($modes)) {
+			my ($rv) = POSIX::mkfifo($file, S_IMODE($modes));
+			return $rv ? 0 : -POSIX::errno();
+		}
+	}
 	syscall(&SYS_mknod,$file,$modes,$dev);
 	return -$!;
 }
@@ -115,22 +141,24 @@ my ($mountpoint) = "";
 $mountpoint = shift(@ARGV) if @ARGV;
 Fuse::main(
 	mountpoint=>$mountpoint,
-	getattr=>\&x_getattr,
-	readlink=>\&x_readlink,
-	getdir=>\&x_getdir,
-	mknod=>\&x_mknod,
-	mkdir=>\&x_mkdir,
-	unlink=>\&x_unlink,
-	rmdir=>\&x_rmdir,
-	symlink=>\&x_symlink,
-	rename=>\&x_rename,
-	link=>\&x_link,
-	chmod=>\&x_chmod,
-	chown=>\&x_chown,
-	truncate=>\&x_truncate,
-	utime=>\&x_utime,
-	open=>\&x_open,
-	read=>\&x_read,
-	write=>\&x_write,
-	statfs=>\&x_statfs,
+	getattr =>"main::x_getattr",
+	readlink=>"main::x_readlink",
+	getdir  =>"main::x_getdir",
+	mknod   =>"main::x_mknod",
+	mkdir   =>"main::x_mkdir",
+	unlink  =>"main::x_unlink",
+	rmdir   =>"main::x_rmdir",
+	symlink =>"main::x_symlink",
+	rename  =>"main::x_rename",
+	link    =>"main::x_link",
+	chmod   =>"main::x_chmod",
+	chown   =>"main::x_chown",
+	truncate=>"main::x_truncate",
+	utime   =>"main::x_utime",
+	open    =>"main::x_open",
+	read    =>"main::x_read",
+	write   =>"main::x_write",
+	statfs  =>"main::x_statfs",
+	threaded=>0,
+	debug => 1,
 );
