@@ -7,8 +7,21 @@ use blib;
 use Fuse;
 use IO::File;
 use POSIX qw(ENOENT ENOSYS EEXIST EPERM O_RDONLY O_RDWR O_APPEND O_CREAT);
-use Fcntl qw(S_ISBLK S_ISCHR S_ISFIFO SEEK_SET);
-require 'syscall.ph'; # for SYS_mknod and SYS_lchown
+use Fcntl qw(S_ISBLK S_ISCHR S_ISFIFO SEEK_SET S_ISREG S_ISFIFO S_IMODE);
+my $can_syscall = eval {
+	require 'syscall.ph'; # for SYS_mknod and SYS_lchown
+};
+if (!$can_syscall && open my $fh, '<', '/usr/include/sys/syscall.h') {
+	my %sys = do { local $/ = undef;
+			<$fh> =~ m/\#define \s+ (\w+) \s+ (\d+)/gxms;
+        };
+	close $fh;
+	if ($sys{SYS_mknod} && $sys{SYS_lchown}) {
+		*SYS_mknod  = sub { $sys{SYS_mknod}  };
+		*SYS_lchown = sub { $sys{SYS_lchown} };
+		$can_syscall = 1;
+	}
+}
 
 sub fixup { return "/tmp/fusetest-" . $ENV{LOGNAME} . shift }
 
@@ -79,6 +92,7 @@ sub x_rename {
 }
 sub x_link { return link(fixup(shift),fixup(shift)) ? 0 : -$! }
 sub x_chown {
+	return -ENOSYS() if ! $can_syscall;
 	my ($fn) = fixup(shift);
 	print "nonexistent $fn\n" unless -e $fn;
 	my ($uid,$gid) = @_;
@@ -102,11 +116,23 @@ sub x_mkdir { my ($name, $perm) = @_; return 0 if mkdir(fixup($name),$perm); ret
 sub x_rmdir { return 0 if rmdir fixup(shift); return -$!; }
 
 sub x_mknod {
+	return -ENOSYS() if ! $can_syscall;
 	# since this is called for ALL files, not just devices, I'll do some checks
 	# and possibly run the real mknod command.
 	my ($file, $modes, $dev) = @_;
 	$file = fixup($file);
-	$! = 0;
+	undef $!;
+	if ($^O eq 'freebsd' || $^O eq 'darwin' || $^O eq 'netbsd') {
+		if (S_ISREG($modes)) {
+			open(FILE, '>', $file) || return -$!;
+			print FILE "";
+			close(FILE);
+			return 0;
+		} elsif (S_ISFIFO($modes)) {
+			my ($rv) = POSIX::mkfifo($file, S_IMODE($modes));
+			return $rv ? 0 : -POSIX::errno();
+		}
+	}
 	syscall(&SYS_mknod,$file,$modes,$dev);
 	return -$!;
 }
